Check for EINVAL from a rename error call in addition to ENOENT because
[rsync/rsync.git] / util.c
diff --git a/util.c b/util.c
index e04d0d7..16d8f6c 100644 (file)
--- a/util.c
+++ b/util.c
@@ -400,7 +400,7 @@ int lock_range(int fd, int offset, int len)
 }
 
 
-static void glob_expand_one(char *s, char **argv, int *argc, int maxargs)
+static void glob_expand_one(char *s, char **argv, int *argc, int maxargs, int sanitize_paths)
 {
 #if !(defined(HAVE_GLOB) && defined(HAVE_GLOB_H))
        if (!*s) s = ".";
@@ -413,7 +413,9 @@ static void glob_expand_one(char *s, char **argv, int *argc, int maxargs)
 
        if (!*s) s = ".";
 
-       argv[*argc] = strdup(s);
+       s = strdup(s);
+       sanitize_path(s);
+       argv[*argc] = s;
 
        memset(&globbuf, 0, sizeof(globbuf));
        glob(argv[*argc], 0, NULL, &globbuf);
@@ -432,7 +434,7 @@ static void glob_expand_one(char *s, char **argv, int *argc, int maxargs)
 #endif
 }
 
-void glob_expand(char *base1, char **argv, int *argc, int maxargs)
+void glob_expand(char *base1, char **argv, int *argc, int maxargs, int sanitize_paths)
 {
        char *s = argv[*argc];
        char *p, *q;
@@ -456,11 +458,11 @@ void glob_expand(char *base1, char **argv, int *argc, int maxargs)
        while ((p = strstr(q,base)) && ((*argc) < maxargs)) {
                /* split it at this point */
                *p = 0;
-               glob_expand_one(q, argv, argc, maxargs);
+               glob_expand_one(q, argv, argc, maxargs, sanitize_paths);
                q = p+strlen(base);
        }
 
-       if (*q && (*argc < maxargs)) glob_expand_one(q, argv, argc, maxargs);
+       if (*q && (*argc < maxargs)) glob_expand_one(q, argv, argc, maxargs, sanitize_paths);
 
        free(s);
        free(base);
@@ -561,23 +563,23 @@ void clean_fname(char *name)
  * While we're at it, remove double slashes and "." components like
  *   clean_fname does(), but DON'T remove a trailing slash because that
  *   is sometimes significant on command line arguments.
- * Return a malloc'ed copy.
+ * Can only shrink paths, so sanitizes in place.
  * Contributed by Dave Dykstra <dwd@bell-labs.com>
  */
 
-char *sanitize_path(char *p)
+void sanitize_path(char *p)
 {
-       char *copy, *copyp;
+       char *start, *sanp;
 
-       copy = (char *) malloc(strlen(p)+1);
-       copyp = copy;
+       start = p;
+       sanp = p;
        while (*p == '/') {
                /* remove leading slashes */
                p++;
        }
        while (*p != '\0') {
                /* this loop iterates once per filename component in p.
* both p (and copyp if the original had a slash) should
                 * always be left pointing after a slash
                 */
                if ((*p == '.') && ((*(p+1) == '/') || (*(p+1) == '\0'))) {
@@ -592,18 +594,18 @@ char *sanitize_path(char *p)
                        p += 2;
                        if (*p == '/')
                                p++;
-                       if (copyp != copy) {
-                               /* back up the copy one level */
-                               --copyp; /* now pointing at slash */
-                               while ((copyp > copy) && (*(copyp - 1) != '/')) {
+                       if (sanp != start) {
+                               /* back up sanp one level */
+                               --sanp; /* now pointing at slash */
+                               while ((sanp > start) && (*(sanp - 1) != '/')) {
                                        /* skip back up to slash */
-                                       copyp--;
+                                       sanp--;
                                }
                        }
                } else {
                        while (1) {
                                /* copy one component through next slash */
-                               *copyp++ = *p++;
+                               *sanp++ = *p++;
                                if ((*p == '\0') || (*(p-1) == '/')) {
                                        while (*p == '/') {
                                                /* skip multiple slashes */
@@ -614,12 +616,11 @@ char *sanitize_path(char *p)
                        }
                }
        }
-       if (copyp == copy) {
+       if (sanp == start) {
                /* ended up with nothing, so put in "." component */
-               *copyp++ = '.';
+               *sanp++ = '.';
        }
-       *copyp = '\0';
-       return(copy);
+       *sanp = '\0';
 }
 
 
@@ -637,6 +638,8 @@ char *push_dir(char *dir, int save)
                getcwd(curr_dir, sizeof(curr_dir)-1);
        }
 
+       if (!dir) return NULL; /* this call was probably just to initialize */
+
        if (chdir(dir)) return NULL;
 
        if (save) {
@@ -561,23 +563,23 @@ void clean_fname(char *name)
        while (*p != '\0') {
                /* this loop iterates once per filename component in p.
-                * both p (and copyp if the original had a slash) should
+                * both p (and sanp if the original had a slash) should
                 * always be left pointing after a slash
                 */